@@ -4,15 +4,14 @@
 
 import base64
 import ConfigParser
-import datetime
 import json
 import os
 import socket
 import StringIO
-import time
 import traceback
 import warnings
 
+from contextlib import contextmanager
 
 from application_cache import ApplicationCache
 from decorators import do_crash_check
@@ -476,8 +475,10 @@ class Marionette(object):
         self.host = host
         self.port = self.local_port = port
         self.bin = bin
+        self.profile = profile
         self.instance = None
         self.session = None
+        self.session_id = None
         self.window = None
         self.runner = None
         self.emulator = None
@@ -512,7 +513,7 @@ class Marionette(object):
                         KeyError):
                     instance_class = geckoinstance.GeckoInstance
             self.instance = instance_class(host=self.host, port=self.port,
-                                           bin=self.bin, profile=profile,
+                                           bin=self.bin, profile=self.profile,
                                            app_args=app_args, symbols_path=symbols_path,
                                            gecko_log=gecko_log)
             self.instance.start()
@@ -528,7 +529,7 @@ class Marionette(object):
                                             binary=emulator_binary,
                                             userdata=emulator_img,
                                             resolution=emulator_res,
-                                            profile=profile,
+                                            profile=self.profile,
                                             adb_path=adb_path,
                                             process_args=process_args)
             self.emulator = self.runner.device
@@ -585,29 +586,18 @@ class Marionette(object):
             s.close()
 
     def wait_for_port(self, timeout=60):
-        starttime = datetime.datetime.now()
-        while datetime.datetime.now() - starttime < datetime.timedelta(seconds=timeout):
-            try:
-                sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-                sock.connect((self.host, self.port))
-                data = sock.recv(16)
-                sock.close()
-                if ':' in data:
-                    time.sleep(5)
-                    return True
-            except socket.error:
-                pass
-            time.sleep(1)
-        return False
+        return MarionetteTransport.wait_for_port(self.host,
+                                                 self.port,
+                                                 timeout=timeout)
 
     @do_crash_check
     def _send_message(self, command, response_key="ok", **kwargs):
-        if not self.session and command != "newSession":
+        if not self.session_id and command != "newSession":
             raise errors.MarionetteException("Please start a session")
 
         message = {"name": command}
-        if self.session:
-            message["sessionId"] = self.session
+        if self.session_id:
+            message["sessionId"] = self.session_id
         if kwargs:
             message["parameters"] = kwargs
 
@@ -632,6 +622,8 @@ class Marionette(object):
                 continue;
 
             break;
+        if not self.session_id:
+            self.session_id = response.get("sessionId", None)
 
         if response_key in response:
             return response[response_key]
@@ -708,6 +700,8 @@ class Marionette(object):
                 raise errors.FrameSendNotInitializedError(message=message, status=status, stacktrace=stacktrace)
             elif status == errors.ErrorCodes.FRAME_SEND_FAILURE_ERROR:
                 raise errors.FrameSendFailureError(message=message, status=status, stacktrace=stacktrace)
+            elif status == errors.ErrorCodes.UNSUPPORTED_OPERATION:
+                raise errors.UnsupportedOperationException(message=message, status=status, stacktrace=stacktrace)
             else:
                 raise errors.MarionetteException(message=message, status=status, stacktrace=stacktrace)
         raise errors.MarionetteException(message=response, status=500)
@@ -725,12 +719,13 @@ class Marionette(object):
         name = None
         crashed = False
         if self.runner:
-            if self.runner.check_for_crashes():
+            if self.runner.check_for_crashes(test_name=self.test_name):
                 returncode = self.emulator.proc.returncode
                 name = 'emulator'
                 crashed = True
         elif self.instance:
-            if self.instance.check_for_crashes():
+            if self.instance.runner.check_for_crashes(
+                    test_name=self.test_name):
                 crashed = True
         if returncode is not None:
             print ('PROCESS-CRASH | %s | abnormal termination with exit code %d' %
@@ -779,20 +774,20 @@ class Marionette(object):
             self.start_session()
             self._reset_timeouts()
 
-    def restart_with_clean_profile(self):
+    def restart(self, clean=False):
         """
         This will terminate the currently running instance, and spawn a new instance
-        with a clean profile.
+        with the same profile and then reuse the session id when creating a session again.
 
         : param prefs: A dictionary whose keys are preference names.
         """
         if not self.instance:
-            raise errors.MarionetteException("enforce_gecko_prefs can only be called " \
+            raise errors.MarionetteException("restart can only be called " \
                                              "on gecko instances launched by Marionette")
         self.delete_session()
-        self.instance.restart()
+        self.instance.restart(clean=clean)
         assert(self.wait_for_port()), "Timed out waiting for port!"
-        self.start_session()
+        self.start_session(session_id=self.session_id)
         self._reset_timeouts()
 
     def absolute_url(self, relative_url):
@@ -803,16 +798,18 @@ class Marionette(object):
         '''
         return "%s%s" % (self.baseurl, relative_url)
 
-    def start_session(self, desired_capabilities=None):
+    def start_session(self, desired_capabilities=None, session_id=None, timeout=60):
         """Create a new Marionette session.
 
         This method must be called before performing any other action.
 
-        :params desired_capabilities: An optional dict of desired
+        :param desired_capabilities: An optional dict of desired
             capabilities.  This is currently ignored.
+        :param timeout: Timeout in seconds for the server to be ready.
 
         :returns: A dict of the capabilities offered."""
-        self.session = self._send_message('newSession', 'value')
+        self.wait_for_port(timeout=timeout)
+        self.session = self._send_message('newSession', 'value', capabilities=desired_capabilities, session_id=session_id)
         self.b2g = 'b2g' in self.session
         return self.session
 
@@ -828,6 +825,7 @@ class Marionette(object):
     def delete_session(self):
         """Close the current session and disconnect from the server."""
         response = self._send_message('deleteSession', 'ok')
+        self.session_id = None
         self.session = None
         self.window = None
         self.client.close()
@@ -882,10 +880,25 @@ class Marionette(object):
         :rtype: string
 
         """
-
         self.window = self._send_message("getWindowHandle", "value")
         return self.window
 
+    def get_window_position(self):
+        """Get the current window's position
+           Return a dictionary with the keys x and y
+           :returns: a dictionary with x and y
+        """
+        return self._send_message("getWindowPosition", "value")
+
+    def set_window_position(self, x, y):
+        """
+           Set the position of the current window
+            :param x: x coordinate for the top left of the window
+            :param y: y coordinate for the top left of the window
+        """
+        response = self._send_message("setWindowPosition", "ok", x=x, y=y)
+        return response
+
     @property
     def title(self):
         '''
@@ -934,20 +947,41 @@ class Marionette(object):
 
     def set_context(self, context):
         '''
-        Sets the context that marionette commands are running in.
+        Sets the context that Marionette commands are running in.
 
         :param context: Context, may be one of the class properties
          `CONTEXT_CHROME` or `CONTEXT_CONTENT`.
 
-        Usage example:
-
-        ::
+        Usage example::
 
           marionette.set_context(marionette.CONTEXT_CHROME)
         '''
         assert(context == self.CONTEXT_CHROME or context == self.CONTEXT_CONTENT)
         return self._send_message('setContext', 'ok', value=context)
 
+    @contextmanager
+    def using_context(self, context):
+        '''
+        Sets the context that Marionette commands are running in using
+        a `with` statement. The state of the context on the server is
+        saved before entering the block, and restored upon exiting it.
+
+        :param context: Context, may be one of the class properties
+         `CONTEXT_CHROME` or `CONTEXT_CONTENT`.
+
+        Usage example::
+
+          with marionette.using_context(marionette.CONTEXT_CHROME):
+              # chrome scope
+              ... do stuff ...
+        '''
+        scope = self._send_message('getContext', 'value')
+        self.set_context(context)
+        try:
+            yield
+        finally:
+            self.set_context(scope)
+
     def switch_to_window(self, window_id):
         '''
         Switch to the specified window; subsequent commands will be directed at the new window.
@@ -1263,14 +1297,15 @@ class Marionette(object):
         NoSuchElementException will be raised.
 
         :param method: The method to use to locate the element; one of: "id",
-         "name", "class name", "tag name", "css selector", "link text",
-         "partial link text" and "xpath". Note that the methods supported in
-         the chrome dom are only "id", "class name", "tag name" and "xpath".
+                       "name", "class name", "tag name", "css selector", "link text",
+                       "partial link text", "xpath", "anon" and "anon attribute".
+                       Note that the "name", "link text" and
+                       "partial link test" methods are not supported in the chrome dom.
         :param target: The target of the search.  For example, if method =
-         "tag", target might equal "div".  If method = "id", target would be
-         an element id.
+                       "tag", target might equal "div".  If method = "id", target would be
+                       an element id.
         :param id: If specified, search for elements only inside the element
-         with the specified id.
+                   with the specified id.
         '''
         kwargs = { 'value': target, 'using': method }
         if id:
@@ -1289,14 +1324,15 @@ class Marionette(object):
         time set by set_search_timeout().
 
         :param method: The method to use to locate the elements; one of:
-         "id", "name", "class name", "tag name", "css selector", "link text",
-         "partial link text" and "xpath". Note that the methods supported in
-         the chrome dom are only "id", "class name", "tag name" and "xpath".
+                       "id", "name", "class name", "tag name", "css selector", "link text",
+                       "partial link text", "xpath", "anon" and "anon attribute".
+                       Note that the "name", "link text" and
+                       "partial link test" methods are not supported in the chrome dom.
         :param target: The target of the search.  For example, if method =
-         "tag", target might equal "div".  If method = "id", target would be
-         an element id.
+                       "tag", target might equal "div".  If method = "id", target would be
+                       an element id.
         :param id: If specified, search for elements only inside the element
-         with the specified id.
+                   with the specified id.
         '''
         kwargs = { 'value': target, 'using': method }
         if id:
@@ -1514,3 +1550,38 @@ class Marionette(object):
         self._send_message("setScreenOrientation", "ok", orientation=orientation)
         if self.emulator:
             self.emulator.screen.orientation = orientation.lower()
+
+    @property
+    def window_size(self):
+        """Get the current browser window size.
+
+        Will return the current browser window size in pixels. Refers to
+        window outerWidth and outerHeight values, which include scroll bars,
+        title bars, etc.
+
+        :returns: dictionary representation of current window width and height
+
+        """
+        return self._send_message("getWindowSize", "value")
+
+    def set_window_size(self, width, height):
+        """Resize the browser window currently in focus.
+
+        The supplied width and height values refer to the window outerWidth
+        and outerHeight values, which include scroll bars, title bars, etc.
+
+        An error will be returned if the requested window size would result
+        in the window being in the maximised state.
+
+        :param width: The width to resize the window to.
+        :param height: The height to resize the window to.
+
+        """
+        self._send_message("setWindowSize", "ok", width=width, height=height)
+
+    def maximize_window(self):
+        """ Resize the browser window currently receiving commands. The action
+        should be equivalent to the user pressing the the maximize button
+        """
+
+        return self._send_message("maximizeWindow", "ok")
